
import imageio.v2 as imageio
import pygfx as gfx

import core.pyka3dmr as core


def map_geometry():
    vertices = []
    normals = []
    indices = []
    texture = []
    offset = 0

    for i in range(core.MAP_WIDTH):
        for j in range(core.MAP_HEIGHT):
            height = core.getBlockHeight(i, j) / 10
            bottom = core.getBlockBottom(i, j) / 10

            if height == 0 and bottom == 0:
                continue

            if bottom != 0:
                vertices.append((i + 0, bottom, j + 0))
                vertices.append((i + 1, bottom, j + 0))
                vertices.append((i + 0, bottom, j + 1))
                vertices.append((i + 1, bottom, j + 1))
                normals.append((0, -1, 0))
                normals.append((0, -1, 0))
                normals.append((0, -1, 0))
                normals.append((0, -1, 0))
                texture.append((1, 1))
                texture.append((0, 1))
                texture.append((1, 0))
                texture.append((0, 0))
                indices.append((offset + 0, offset + 1, offset + 3))
                indices.append((offset + 0, offset + 3, offset + 2))
                offset += 4

            vertices.append((i + 0, bottom + height, j + 0))
            vertices.append((i + 1, bottom + height, j + 0))
            vertices.append((i + 0, bottom + height, j + 1))
            vertices.append((i + 1, bottom + height, j + 1))
            normals.append((0, 1, 0))
            normals.append((0, 1, 0))
            normals.append((0, 1, 0))
            normals.append((0, 1, 0))
            texture.append((1, 1))
            texture.append((0, 1))
            texture.append((1, 0))
            texture.append((0, 0))
            indices.append((offset + 0, offset + 3, offset + 1))
            indices.append((offset + 0, offset + 2, offset + 3))
            offset += 4

            vertices.append((i + 0, bottom, j + 0))
            vertices.append((i + 1, bottom, j + 0))
            vertices.append((i + 0, bottom + height, j + 0))
            vertices.append((i + 1, bottom + height, j + 0))
            normals.append((0, 0, -1))
            normals.append((0, 0, -1))
            normals.append((0, 0, -1))
            normals.append((0, 0, -1))
            texture.append((1, height))
            texture.append((0, height))
            texture.append((1, 0))
            texture.append((0, 0))
            indices.append((offset + 0, offset + 3, offset + 1))
            indices.append((offset + 0, offset + 2, offset + 3))
            offset += 4

            vertices.append((i + 0, bottom, j + 1))
            vertices.append((i + 1, bottom, j + 1))
            vertices.append((i + 0, bottom + height, j + 1))
            vertices.append((i + 1, bottom + height, j + 1))
            normals.append((0, 0, 1))
            normals.append((0, 0, 1))
            normals.append((0, 0, 1))
            normals.append((0, 0, 1))
            texture.append((1, height))
            texture.append((0, height))
            texture.append((1, 0))
            texture.append((0, 0))
            indices.append((offset + 0, offset + 1, offset + 3))
            indices.append((offset + 0, offset + 3, offset + 2))
            offset += 4

            vertices.append((i + 0, bottom, j + 0))
            vertices.append((i + 0, bottom, j + 1))
            vertices.append((i + 0, bottom + height, j + 0))
            vertices.append((i + 0, bottom + height, j + 1))
            normals.append((-1, 0, 0))
            normals.append((-1, 0, 0))
            normals.append((-1, 0, 0))
            normals.append((-1, 0, 0))
            texture.append((1, height))
            texture.append((0, height))
            texture.append((1, 0))
            texture.append((0, 0))
            indices.append((offset + 0, offset + 1, offset + 3))
            indices.append((offset + 0, offset + 3, offset + 2))
            offset += 4

            vertices.append((i + 1, bottom, j + 0))
            vertices.append((i + 1, bottom, j + 1))
            vertices.append((i + 1, bottom + height, j + 0))
            vertices.append((i + 1, bottom + height, j + 1))
            normals.append((1, 0, 0))
            normals.append((1, 0, 0))
            normals.append((1, 0, 0))
            normals.append((1, 0, 0))
            texture.append((1, height))
            texture.append((0, height))
            texture.append((1, 0))
            texture.append((0, 0))
            indices.append((offset + 0, offset + 3, offset + 1))
            indices.append((offset + 0, offset + 2, offset + 3))
            offset += 4

    return gfx.Geometry(indices=indices, positions=vertices, normals=normals, texcoords=texture)


def init_map(scene):
    im = imageio.imread("wall.png")
    tex = gfx.Texture(im, dim=2).get_view(
        filter="linear", address_mode="repeat")
    scene.add(gfx.Mesh(
        map_geometry(),
        gfx.MeshPhongMaterial(map=tex, side="front"),
    ))

    ground = gfx.Mesh(
        gfx.plane_geometry(core.MAP_WIDTH, core.MAP_HEIGHT),
        gfx.MeshPhongMaterial(color="#552200"),
    )
    rot = gfx.linalg.Quaternion().set_from_euler(
        gfx.linalg.Euler(3.141592 / 2, 0, 0))
    ground.rotation.multiply(rot)
    ground.position.set(core.MAP_WIDTH / 2, 0, core.MAP_HEIGHT / 2)
    scene.add(ground)
